cwlVersion: v1.0
class: Workflow
label: CNV_FPS

requirements:
  ScatterFeatureRequirement: {}
  MultipleInputFeatureRequirement: {}
  SubworkflowFeatureRequirement: {}

$namespaces:
  s: https://schema.org/

inputs:
  fastq1: {type: 'File[]', doc: "FASTQ file"}
  fastq2: {type: 'File[]', doc: "FASTQ file"}
  reference_fasta: {type: File, doc: "reference genome"}
  bed: {type: File, doc: "capture kit"}
  samples: {type: File, doc: "TXT file mapping samples per batch"}
  threads_fastqc: {type: 'int?', doc: "number of threads for fastqc"}
  threads_fastp: {type: 'int?', doc: "number of threads for fastp"}
  threads_bwa_mem: {type: 'int?', doc: "number of threads for bwa_mem"}
  threads_samtools: {type: 'int?', doc: "number of threads for samtools"}
  threads_gridss: {type: 'int?', doc: "number of threads for GRIDSS"}
  read_group: {type: string, doc: "parse header which should correlate to FASTQ files"}
  min_mapping_quality: {type: int, doc: "skip alignments with MAPQ smaller than this minimum"}
  bits_set: {type: int, doc: "skip output alignments with this bits set"}
  manta_exome: {type: boolean, doc: "disable all high depth filters"}
  manta_min_len: {type: string, doc: "minimum lenght for MANTA's filter"}
  manta_max_len: {type: string, doc: "maximum lenght for MANTA's filter"}
  manta_min_q: {type: string, doc: "minimum quality for MANTA's filter"}
  blacklist: {type: 'File?', doc: "BED file containing regions to ignore for GRIDSS"}
  gridss_min_len: {type: string, doc: "minimum lenght for GRIDSS's filter"}
  gridss_max_len: {type: string, doc: "maximum lenght for GRIDSS's filter"}
  gridss_min_q: {type: string, doc: "minimum quality for GRIDSS's filter"}
  exomeDepth_min_len: {type: string, doc: "minimum lenght for EXOME_DEPTH's filter"}
  exomeDepth_max_len: {type: string, doc: "maximum lenght for EXOME_DEPTH's filter"}
  exomeDepth_min_q: {type: string, doc: "minimum quality for EXOME_DEPTH's filter"}
  chromosome: {type: string, doc: "chromosome for targeted sequencing for CODEX"}
  codex_min_len: {type: string, doc: "minimum lenght for CODEX's filter"}
  codex_max_len: {type: string, doc: "maximum lenght for CODEX's filter"}
  codex_min_lratio: {type: string, doc: "minimum lratio for CODEX's filter"}

outputs:
  fastqc_raw_zip: {type: 'File[]', outputSource: trimmed_fastq/fastqc_raw_zip}
  fastqc_raw_html: {type: 'File[]', outputSource: trimmed_fastq/fastqc_raw_html}
  html_report: {type: 'File[]', outputSource: trimmed_fastq/html_report}
  json_report: {type: 'File[]', outputSource: trimmed_fastq/json_report}
  fastqc_paired_zip: {type: 'File[]', outputSource: trimmed_fastq/fastqc_paired_zip}
  fastqc_paired_html: {type: 'File[]', outputSource: trimmed_fastq/fastqc_paired_html}
  output_bam_filtering: {type: 'File[]', outputSource: bam_filtering/output}
  output_manta: {type: 'File[]', outputSource: cnv_manta/output}
  output_gridss: {type: 'File[]', outputSource: cnv_gridss/output}
  output_exomedepth: {type: 'File[]', outputSource: cnv_exomedepth/output}
  output_codex: {type: 'File[]', outputSource: cnv_codex/output}

steps:
  trimmed_fastq:
    id: trimmed-fastq
    run: subworkflows/trimmed_fastq.cwl
    in:
      fastq1: fastq1
      fastq2: fastq2
      threads_fastqc: threads_fastqc
      threads_fastp: threads_fastp
    out: [fastqc_raw_zip, fastqc_raw_html, paired_R1, paired_R2, unpaired_R1, unpaired_R2, html_report, json_report, fastqc_paired_zip, fastqc_paired_html]

  bwa_index:
    id: bwa-index
    run: subworkflows/bwa_index.cwl
    in:
      input: reference_fasta
    out: [output]

  bwa_mem:
    id: bwa-mem
    run: subworkflows/bwa_mem.cwl
    in:
      paired_R1:
        source: trimmed_fastq/paired_R1
      paired_R2:
        source: trimmed_fastq/paired_R2
      unpaired_R1:
        source: trimmed_fastq/unpaired_R1
      unpaired_R2:
        source: trimmed_fastq/unpaired_R2
      reference_fasta:
        source: bwa_index/output
      threads: threads_bwa_mem
      read_group: read_group
    scatter:
      - paired_R1
      - paired_R2
      - unpaired_R1
      - unpaired_R2
    scatterMethod: dotproduct
    out: [output_paired, output_unpairedR1, output_unpairedR2]

  samtools_view_sam2bam:
    id: samtools-view-sam2bam
    run: subworkflows/samtools_view_sam2bam.cwl
    in:
      paired:
        source: bwa_mem/output_paired
      unpaired_R1:
        source: bwa_mem/output_unpairedR1
      unpaired_R2:
        source: bwa_mem/output_unpairedR2
      threads: threads_samtools
    out: [output_paired, output_unpairedR1, output_unpairedR2]

  samtools_sort:
    id: samtools-sort
    run: subworkflows/samtools_sort.cwl
    in:
      paired:
        source: samtools_view_sam2bam/output_paired
      unpaired_R1:
        source: samtools_view_sam2bam/output_unpairedR1
      unpaired_R2:
        source: samtools_view_sam2bam/output_unpairedR2
      threads: threads_samtools
    out: [output_paired, output_unpairedR1, output_unpairedR2]

  samtools_merge:
    run: tools/samtools_merge.cwl
    in:
      paired:
        source: samtools_sort/output_paired
      unpaired_R1:
        source: samtools_sort/output_unpairedR1
      unpaired_R2:
        source: samtools_sort/output_unpairedR2
      threads: threads_samtools
    scatter:
      - paired
      - unpaired_R1
      - unpaired_R2
    scatterMethod: dotproduct
    out: [output]

  samtools_index:
    run: tools/samtools_index.cwl
    in:
      input:
        source: samtools_merge/output
    scatter: input
    out: [output]

  picard_markduplicates:
    id: picard-markduplicates
    run: subworkflows/picard_markduplicates.cwl
    in:
      input:
        source: samtools_index/output
    scatter: input
    out: [alignments, metrics]

  bam_filtering:
    id: bam-filtering
    run: subworkflows/bam_filtering.cwl
    in:
      input:
        source: picard_markduplicates/alignments
      min_mapping_quality: min_mapping_quality
      bits_set: bits_set
      threads: threads_samtools
    scatter: input
    out: [output]

  cnv_manta:
    id: cnv-manta
    run: subworkflows/cnv_manta.cwl
    in:
      bams:
        source: bam_filtering/output
      samples: samples
      reference_genome:
        source: bwa_index/output
      bed: bed
      exome: manta_exome
      min_len: manta_min_len
      max_len: manta_max_len
      min_q: manta_min_q
    out: [output]

  cnv_gridss:
    id: cnv-gridss
    run: subworkflows/cnv_gridss.cwl
    in:
      bams:
        source: bam_filtering/output
      reference_genome:
        source: bwa_index/output
      blacklist: blacklist
      threads: threads_gridss
      min_len: gridss_min_len
      max_len: gridss_max_len
      min_q: gridss_min_q
    out: [output]

  cnv_exomedepth:
    id: cnv-exomedepth
    run: subworkflows/cnv_exome_depth.cwl
    in:
      bams:
        source: bam_filtering/output
      samples: samples
      reference_genome:
        source: bwa_index/output
      min_len: exomeDepth_min_len
      max_len: exomeDepth_max_len
      min_q: exomeDepth_min_q
    out: [output]

  cnv_codex:
    id: cnv-codex
    run: subworkflows/cnv_codex.cwl
    in:
      bams:
        source: bam_filtering/output
      samples: samples
      bed: bed
      chromosome: chromosome
      min_len: codex_min_len
      max_len: codex_max_len
      min_lratio: codex_min_lratio
    out: [output]

s:author:
  - class: s:Person
    s:identifier: https://orcid.org/0000-0003-4929-1219
    s:email: mailto:laura.rodriguez@bsc.es
    s:name: Laura Rodríguez-Navas
  - class: s:Person
    s:identifier: https://orcid.org/0000-0001-7672-7335
    s:email: mailto:daniel.lopez.lopez@juntadeandalucia.es
    s:name: Daniel López-López

s:codeRepository: https://gitlab.bsc.es/lrodrig1/structuralvariants_poc.git # TODO move to INB general project
s:dateCreated: "2020-10-08"
s:license: https://spdx.org/licenses/Apache-2.0
